/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourcemodel.h"
#include "resourceguiitemfactory.h"
#include "abstractresourceguiitem.h"

#include <QtCore/QUrl>
#include <QtCore/QList>
#include <QtCore/QMimeData>

#include <KUrl>
#include <KDebug>
#include <kcategorizedsortfilterproxymodel.h>
#include <KIcon>
#include <KLocale>

#include <nepomuk/resource.h>
#include <nepomuk/class.h>

#include <Soprano/Vocabulary/RDFS>


Q_DECLARE_METATYPE( Nepomuk::AbstractResourceGuiItem* )

class Nepomuk::ResourceModel::Private
{
public:
    Private( ResourceModel* parent )
        : q( parent ) {
    }

    // caching the gui items
    ResourceGuiItemFactory factory;
    AbstractResourceGuiItem::DisplayMode displayMode;
    QHash<QString, AbstractResourceGuiItem*> itemCache;

    AbstractResourceGuiItem* itemForResource( const QUrl& url );
    void _k_dataChanged( const QUrl& uri );

private:
    ResourceModel* q;
};


Nepomuk::AbstractResourceGuiItem* Nepomuk::ResourceModel::Private::itemForResource( const QUrl& url )
{
    Q_ASSERT( url.isValid() );
    QHash<QString, AbstractResourceGuiItem*>::const_iterator it = itemCache.constFind( url.toString() );
    if ( it != itemCache.constEnd() ) {
        return it.value();
    }
    else {
        AbstractResourceGuiItem* item = factory.guiItemForResource( url );
        connect( item, SIGNAL( dataChanged( const QUrl& ) ),
                 q, SLOT( _k_dataChanged( const QUrl& ) ) );
        itemCache.insert( url.toString(), item );
        return item;
    }
}


void Nepomuk::ResourceModel::Private::_k_dataChanged( const QUrl& uri )
{
    kDebug() << uri;
    QModelIndex index = q->indexForResource( uri );
    Q_ASSERT( index.isValid() );
    emit q->dataChanged( index, index );
}



Nepomuk::ResourceModel::ResourceModel( QObject* parent )
    : QAbstractItemModel( parent ),
      d( new Private(this) )
{
}


Nepomuk::ResourceModel::~ResourceModel()
{
    delete d;
}


int Nepomuk::ResourceModel::columnCount( const QModelIndex& ) const
{
    return 1;
}


QVariant Nepomuk::ResourceModel::data( const QModelIndex& index, int role ) const
{
    QUrl uri = resourceForIndex( index );
    if( uri.isValid() ) {
        switch( role ) {
        case Qt::DisplayRole:
            return Resource( uri ).genericLabel();

        case Qt::DecorationRole:
            return KIcon( Resource( uri ).genericIcon() );

        case Qt::ToolTipRole:
            return KUrl( uri ).prettyUrl();

        case KCategorizedSortFilterProxyModel::CategorySortRole:
            // FIXME: sort files before other stuff and so on

        case KCategorizedSortFilterProxyModel::CategoryDisplayRole: {
            Nepomuk::Resource res( uri );
            Q_ASSERT( !res.resourceType().isEmpty() );
            Nepomuk::Types::Class c( res.resourceType() );
            QString cat = c.label();
            if ( cat.isEmpty() ) {
                cat = c.name();
            }
            if ( c.uri() == Soprano::Vocabulary::RDFS::Resource() || cat.isEmpty() ) {
                cat = i18n( "Miscellaneous" );
            }

            return cat;
        }

        case ResourceGuiItemRole:
            return qVariantFromValue( d->itemForResource( uri ) );

        case ResourceUriRole:
            return uri;

        default:
            return d->itemForResource( uri )->data( uri, role );
        }
    }
    return QVariant();
}


QModelIndex Nepomuk::ResourceModel::parent( const QModelIndex& ) const
{
    return QModelIndex();
}


Qt::ItemFlags Nepomuk::ResourceModel::flags( const QModelIndex& index ) const
{
    if ( index.isValid() ) {
        return QAbstractItemModel::flags( index ) | Qt::ItemIsDragEnabled;
    }
    else {
        return QAbstractItemModel::flags( index );
    }
}


QMimeData* Nepomuk::ResourceModel::mimeData( const QModelIndexList& indexes ) const
{
    KUrl::List uris;
    foreach ( const QModelIndex& index, indexes ) {
        if (index.isValid()) {
            uris << index.data( ResourceUriRole ).toUrl();
        }
    }

    QMimeData* mimeData = new QMimeData();
    uris.populateMimeData( mimeData );

    QByteArray data;
    QDataStream s( &data, QIODevice::WriteOnly );
    s << uris;
    mimeData->setData( mimeTypes().first(), data );

    return mimeData;
}


//bool dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent );
QStringList Nepomuk::ResourceModel::mimeTypes() const
{
    return( QStringList()
            << QLatin1String( "application/x-nepomuk-resource-uri" )
            << KUrl::List::mimeDataTypes() );
}

#include "resourcemodel.moc"
