/*
   Copyright (C) 2009 by Dupuy Mathieu <mdupuy at mandriva.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <QFile>
#include <QDir>
#include <QRegExp>
#include <QByteArray>
#include <QUrl>

#include <KUrl>
#include <KTempDir>
#include <KProcess>

#include <KDebug>

#include <Soprano/Node>
#include <Soprano/Vocabulary/RDF>

#include "tmo.h"
#include "nfo.h"

static const char* ext = ".n4";
static const char* documentsDirName = "Exported_Documents";

//TODO: handle multiple file with the same name
//TODO: the temp file/dir KDE api (to avoid document dir erasing at extraction in the same dir)

void    taskExport( const KUrl& archivePath )
{
  QDir::setCurrent( archivePath.directory() );
  const QString name = archivePath.fileName().remove("tar.bz2");

  //creating a synchrone process that export all triplets related to files using soprano export
  KProcess sopranocmdExport;
  sopranocmdExport << "sopranocmd" << "--dbus" << "org.kde.NepomukStorage" << "--model" << "main" << "export" << QString( "describe ?r where { ?r a %1 . }").arg(Soprano::Node::resourceToN3( Nepomuk::Vocabulary::TMO::Task() )) << name + ext + ".tmp";
  if (sopranocmdExport.execute() != 0) {
    qFatal( "sopranocmd export failed" );
  }

  // Open the created file, that contains exported triplets
  QFile in( name + ext + ".tmp" );
  if ( in.open( QFile::ReadOnly ) == false ) {
    qFatal( "Unable to open the exported statements file" );
  }

  if ( QDir().exists( name ) )  // if archive dir already exist
    if ( KTempDir::removeDir( name ) == false ) { // we delete it
      qFatal( "Export directories already exist and unable to remove it" );
    }
  QDir().mkpath( name + "/" + documentsDirName );
  QDir::setCurrent( name );     // descending into the archive dir

  //Create or truncate a file that will contain modified exported triplets
  QFile out( name + ext );
  if ( out.open( QFile::WriteOnly | QFile::Truncate ) == false ) {
    qFatal( "Unable to create temp file" );
  }
  QDir::setCurrent( documentsDirName );     // descending into the documents dir

  kDebug() << QDir::currentPath();

  //modification
  QByteArray line;
  QRegExp t("<file://((/[^>]+/)([^>]+))>");  // super-regexp that match separately path and file name;
  QSet<QString>    files;                       // set of already copied files
  do {
    line = in.readLine();
    if ( t.indexIn( line ) != -1 ) // if a '<file://' pattern is found
      {
        //QString::fromUtf8 (QByteArray::fromPercentEncoding( t.cap(3).toLocal8Bit() ));
        QString filePath = QUrl::fromEncoded( t.cap(1).toAscii() ).toLocalFile(); // we retrieve it
        if ( !files.contains( t.cap(1) ))                                         // if not already handled
          {
            QString fileName =  QUrl::fromEncoded( t.cap(3).toAscii() ).toLocalFile(); // we retrieve it
            kDebug() << "file handled:" << filePath; // we make a link into the document dir
            if ( QFileInfo(fileName).exists() ) {        // if a file with the same name already exists
              static QHash<QString, unsigned int> occurences;
              if ( occurences.contains( fileName ) == false )
                occurences[fileName] = 1;
              fileName.insert( fileName.indexOf(".") , QString::number( ++(occurences[fileName]) ));
            }
            QFile::link( filePath, fileName );
            files.insert( t.cap(1) );
          }
        out.write( line.replace( t.cap(2), "" ));
      }
    else {
      out.write( line );
    }
  } while ( !line.isNull() );

  in.close();
  out.close();
  QDir::setCurrent("../..");
  in.remove();

  KProcess createArchive;
  createArchive << "tar" << "cjf" << archivePath.path() + ".tar.bz2" << "-h" << "--remove-files" << name;
  kDebug() << createArchive.program();
  if ( createArchive.execute() != 0 ) {
    qFatal( "Create archive failed " );
  }

  //removing the whole export dir
  //KTempDir::removeDir( name );
}

void    taskImport( const KUrl& archivePath, const KUrl& location )
{
  if ( QDir::setCurrent( location.path() ) == false ) {
      qFatal( "invalid location" );
    }

  KProcess extract;
  extract << "tar" << "xf" << archivePath.path();
  if ( extract.execute() != 0 ) {
    qFatal( "Extracting the specified archive failed" );
  }

  const QString name = archivePath.fileName().remove(".tar.bz2");
  QDir::setCurrent( name ); //descending into namme

  QFile in( name + ext );
  if ( in.open( QFile::ReadOnly ) == false ) {
    qFatal( "unable to open statement file ; maybe not a valid archive" );
  }

  QFile out( name + ext + ".mod" );
  if ( out.open( QFile::WriteOnly ) == false ) {
      qFatal( "unable to open a temporary file" );
    }

  QDir::setCurrent( documentsDirName );
  QRegExp       t("<file://([^>]+)>"); // regexp for matching the whole <file://> line
  const QByteArray rdfType = Soprano::Node::resourceToN3( Soprano::Vocabulary::RDF::type() ).toAscii();
  const QByteArray nfoFileDataObject = Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::FileDataObject() ).toAscii();
  QByteArray    line;
  do {
    line = in.readLine();
    if ( t.indexIn( line ) != -1 ) // if a '<file://' pattern is found
      {
        QByteArray filePath = "<file://" + QFileInfo( t.cap(1) ).absoluteFilePath().toAscii() + ">";
        kDebug() << filePath;
        out.write( line.replace( t.cap(0).toAscii(), filePath ) );
        out.write( QByteArray( filePath ).append(" ").append(rdfType).append(" ").append(nfoFileDataObject).append( " .\n" ) );
      }
    else {
      out.write(line);
    }
  } while ( !line.isNull() );
  QDir::setCurrent( ".." );

  in.close();
  out.close();
  in.remove();

  KProcess sopranocmdImport;
  sopranocmdImport << "sopranocmd" << "--dbus" << "org.kde.NepomukStorage" << "--model" << "main" << "import" << name + ext + ".mod";
  if (sopranocmdImport.execute() != 0) {
    qFatal( "sopranocmd import failed" );
  }
  out.remove();
}
